@@ -1,7 +1,7 @@
 /*
 	NatFeat host CD-ROM access, Linux CD-ROM driver
 
-	ARAnyM (C) 2003 Patrice Mandin
+	ARAnyM (C) 2003-2005 Patrice Mandin
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -18,11 +18,11 @@
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#include <linux/cdrom.h>
-
 #include <SDL_cdrom.h>
 #include <SDL_endian.h>
 
+#include <linux/cdrom.h>
+
 #include "sysdeps.h"
 #include "cpu_emulation.h"
 #include "parameters.h"
@@ -53,7 +53,7 @@ struct atari_cdrom_audioctrl
 typedef struct {
     unsigned char	audiostatus;
     unsigned char	mcn[23];
-} atari_mcn_t __attribute__((packed));
+} __attribute__((packed)) atari_mcn_t;
 
 /*--- Public functions ---*/
 
@@ -80,27 +80,6 @@ CdromDriverLinux::~CdromDriverLinux()
 
 /*--- Private functions ---*/
 
-void CdromDriverLinux::ScanDrives()
-{
-	int i;
-
-	D(bug(NFCD_NAME "ScanDrives()"));
-
-	drives_mask = 0;
-	for (i='A'; i<='Z'; i++) {
-		/* Check if there is a valid filename */
-		if (bx_options.nfcdroms[i-'A'].physdevtohostdev < 0) {
-			continue;
-		}
-
-		/* Check it is a device file, cdrom type */
-
-		/* Add to MetaDOS CD-ROM drives */
-		drives_mask |= 1<<(i-'A');
-		D(bug(NFCD_NAME "ScanDrives(): physical device %c added", i));
-	}
-}
-
 int CdromDriverLinux::OpenDrive(memptr device)
 {
 	int drive;
@@ -208,6 +187,7 @@ int32 CdromDriverLinux::cd_read(memptr device, memptr buffer, uint32 first, uint
 
 int32 CdromDriverLinux::cd_status(memptr device, memptr ext_status)
 {
+	DUNUSED(ext_status);
 	int drive, errorcode, mediachanged;
 	unsigned long status;
 
@@ -441,6 +421,7 @@ int32 CdromDriverLinux::cd_ioctl(memptr device, uint16 opcode, memptr buffer)
 
 int32 CdromDriverLinux::cd_startaudio(memptr device, uint32 dummy, memptr buffer)
 {
+	DUNUSED(dummy);
 	int drive, errorcode;
 	struct cdrom_ti	track_index;
 	metados_bos_tracks_t	*atari_track_index;
@@ -477,6 +458,7 @@ int32 CdromDriverLinux::cd_stopaudio(memptr device)
 
 int32 CdromDriverLinux::cd_setsongtime(memptr device, uint32 dummy, uint32 start_msf, uint32 end_msf)
 {
+	DUNUSED(dummy);
 	int drive, errorcode;
 	struct cdrom_msf audio_bloc;
 
@@ -502,6 +484,7 @@ int32 CdromDriverLinux::cd_setsongtime(memptr device, uint32 dummy, uint32 start
 
 int32 CdromDriverLinux::cd_gettoc(memptr device, uint32 dummy, memptr buffer)
 {
+	DUNUSED(dummy);
 	int drive, i, numtracks, errorcode;
 	struct cdrom_tochdr tochdr;
 	struct cdrom_tocentry tocentry;
@@ -657,3 +640,7 @@ int32 CdromDriverLinux::cd_discinfo(memptr device, memptr buffer)
 	CloseDrive(drive);
 	return 0;
 }
+
+/*
+vim:ts=4:sw=4:
+*/
