@@ -30,6 +30,9 @@
 #include "memory.h"
 #include "newcpu.h"
 #include "debug.h"
+#ifdef USE_JIT
+# include "compiler/compemu.h"
+#endif
 
 #define DBG_MMU_VERBOSE	1
 #define DBG_MMU_SANITY	1
@@ -112,11 +115,11 @@ static inline int mmu_match_ttr(uaecptr addr, int super, int data)
 
 	if (data) {
 		res = mmu_do_match_ttr(regs.dtt0, addr, super);
-		if (res != TTR_NO_MATCH)
+		if (res == TTR_NO_MATCH)
 			res = mmu_do_match_ttr(regs.dtt1, addr, super);
 	} else {
 		res = mmu_do_match_ttr(regs.itt0, addr, super);
-		if (res != TTR_NO_MATCH)
+		if (res == TTR_NO_MATCH)
 			res = mmu_do_match_ttr(regs.itt1, addr, super);
 	}
 	return res;
@@ -275,7 +278,7 @@ void mmu_dump_tables(void)
 
 static uaecptr REGPARAM2 mmu_lookup_pagetable(uaecptr addr, int super, int write);
 
-static __always_inline int mmu_get_fc(bool super, bool data)
+static ALWAYS_INLINE int mmu_get_fc(bool super, bool data)
 {
 	return (super ? 4 : 0) | (data ? 1 : 2);
 }
@@ -381,7 +384,7 @@ static uaecptr mmu_fill_atc_l2(uaecptr addr, int super, int data, int write,
 	return desc;
 }
 
-static __always_inline bool
+static ALWAYS_INLINE bool
 mmu_fill_atc_l1(uaecptr addr, int super, int data, int write,
 				struct mmu_atc_line *l1)
 {
