@@ -187,8 +187,8 @@ PUBLIC void FFPU dump_registers(const char * str)
 
 	sprintf(temp_str, "%s: %.04f, %.04f, %.04f, %.04f, %.04f, %.04f, %.04f, %.04f\n",
 		str,
-		get_register(0), get_register(1), get_register(2), get_register(3),
-		get_register(4), get_register(5), get_register(6), get_register(7) );
+		fpu_get_register(0), fpu_get_register(1), fpu_get_register(2), fpu_get_register(3),
+		fpu_get_register(4), fpu_get_register(5), fpu_get_register(6), fpu_get_register(7) );
 	
 	fpu_debug((temp_str));
 }
@@ -216,9 +216,7 @@ PUBLIC void FFPU dump_registers(const char *)
 {
 }
 
-PUBLIC void FFPU dump_first_bytes(uae_u8 *, uae_s32)
-{
-}
+#define dump_first_bytes(a,b)
 
 #endif
 
@@ -240,10 +238,10 @@ PRIVATE inline fpu_register FFPU round_to_nearest(fpu_register const & x)
 
 PRIVATE inline bool FFPU do_isnan(fpu_register const & r)
 {
-	uae_u32 * p = (uae_u32 *)&r;
-	if ((p[FHI] & 0x7FF00000) == 0x7FF00000) {
+	fpu_register_parts const *p = (fpu_register_parts const *)&r;
+	if ((p->parts[FHI] & 0x7FF00000) == 0x7FF00000) {
 		// logical or is faster here.
-		if ((p[FHI] & 0x000FFFFF) || p[FLO]) {
+		if ((p->parts[FHI] & 0x000FFFFF) || p->parts[FLO]) {
 			return true;
 		}
 	}
@@ -256,8 +254,8 @@ PRIVATE inline bool FFPU do_isnan(fpu_register const & r)
 
 PRIVATE inline bool FFPU do_isinf(fpu_register const & r)
 {
-	uae_u32 * p = (uae_u32 *)&r;
-	if (((p[FHI] & 0x7FF00000) == 0x7FF00000) && p[FLO] == 0) {
+	fpu_register_parts const *p = (fpu_register_parts const *)&r;
+	if ((p->parts[FHI] & 0x7FF00000) == 0x7FF00000 && p->parts[FLO] == 0) {
 		return true;
 	}
 	return false;
@@ -269,8 +267,8 @@ PRIVATE inline bool FFPU do_isinf(fpu_register const & r)
 
 PRIVATE inline bool FFPU do_isneg(fpu_register const & r)
 {
-	uae_u32 * p = (uae_u32 *)&r;
-	return ((p[FHI] & 0x80000000) != 0);
+	fpu_register_parts const *p = (fpu_register_parts const *)&r;
+	return ((p->parts[FHI] & 0x80000000) != 0);
 }
 
 #ifndef HAVE_ISZERO
@@ -279,8 +277,8 @@ PRIVATE inline bool FFPU do_isneg(fpu_register const & r)
 
 PRIVATE inline bool FFPU do_iszero(fpu_register const & r)
 {
-	uae_u32 * p = (uae_u32 *)&r;
-	return (((p[FHI] & 0x7FF00000) == 0) && p[FLO] == 0);
+	fpu_register_parts const *p = (fpu_register_parts const *)&r;
+	return (((p->parts[FHI] & 0x7FF00000) == 0) && p->parts[FLO] == 0);
 }
 
 // May be optimized for particular processors
@@ -316,75 +314,75 @@ PRIVATE inline void FFPU get_source_flags(fpu_register const & r)
 
 PRIVATE inline void FFPU make_nan(fpu_register & r)
 {
-	uae_u32 * const p = (uae_u32 *)&r;
-	p[FLO] = 0xffffffff;
-	p[FHI] = 0x7fffffff;
+	fpu_register_parts *p = (fpu_register_parts *)&r;
+	p->parts[FLO] = 0xffffffff;
+	p->parts[FHI] = 0x7fffffff;
 }
 
 PRIVATE inline void FFPU make_zero_positive(fpu_register & r)
 {
-	uae_u32 * const p = (uae_u32 *)&r;
-	p[FLO] = p[FHI] = 0;
+	fpu_register_parts *p = (fpu_register_parts *)&r;
+	p->parts[FLO] = p->parts[FHI] = 0;
 }
 
 PRIVATE inline void FFPU make_zero_negative(fpu_register & r)
 {
-	uae_u32 * const p = (uae_u32 *)&r;
-	p[FLO] = 0;
-	p[FHI] = 0x80000000;
+	fpu_register_parts *p = (fpu_register_parts *)&r;
+	p->parts[FLO] = 0;
+	p->parts[FHI] = 0x80000000;
 }
 
 PRIVATE inline void FFPU make_inf_positive(fpu_register & r)
 {
-	uae_u32 * const p = (uae_u32 *)&r;
-	p[FLO] = 0;
-	p[FHI] = 0x7FF00000;
+	fpu_register_parts *p = (fpu_register_parts *)&r;
+	p->parts[FLO] = 0;
+	p->parts[FHI] = 0x7FF00000;
 }
 
 PRIVATE inline void FFPU make_inf_negative(fpu_register & r)
 {
-	uae_u32 * const p = (uae_u32 *)&r;
-	p[FLO] = 0;
-	p[FHI] = 0xFFF00000;
+	fpu_register_parts *p = (fpu_register_parts *)&r;
+	p->parts[FLO] = 0;
+	p->parts[FHI] = 0xFFF00000;
 }
 
 PRIVATE inline void FFPU fast_scale(fpu_register & r, int add)
 {
-	uae_u32 * const p = (uae_u32 *)&r;
-	int exp = (p[FHI] & 0x7FF00000) >> 20;
+	fpu_register_parts *p = (fpu_register_parts *)&r;
+	int exp = (p->parts[FHI] & 0x7FF00000) >> 20;
 	// TODO: overflow flags
 	exp += add;
 	if(exp >= 2047) {
 		make_inf_positive(r);
 	} else if(exp < 0) {
 		// keep sign (+/- 0)
-		p[FHI] &= 0x80000000;
+		p->parts[FHI] &= 0x80000000;
 	} else {
-		p[FHI] = (p[FHI] & 0x800FFFFF) | ((uae_u32)exp << 20);
+		p->parts[FHI] = (p->parts[FHI] & 0x800FFFFF) | ((uae_u32)exp << 20);
 	}
 }
 
 PRIVATE inline fpu_register FFPU fast_fgetexp(fpu_register const & r)
 {
-	uae_u32 * const p = (uae_u32 *)&r;
-	int exp = (p[FHI] & 0x7FF00000) >> 20;
+	fpu_register_parts const *p = (fpu_register_parts const *)&r;
+	int exp = (p->parts[FHI] & 0x7FF00000) >> 20;
 	return( exp - 1023 );
 }
 
 // Normalize to range 1..2
 PRIVATE inline void FFPU fast_remove_exponent(fpu_register & r)
 {
-	uae_u32 * const p = (uae_u32 *)&r;
-	p[FHI] = (p[FHI] & 0x800FFFFF) | 0x3FF00000;
+	fpu_register_parts *p = (fpu_register_parts *)&r;
+	p->parts[FHI] = (p->parts[FHI] & 0x800FFFFF) | 0x3FF00000;
 }
 
 // The sign of the quotient is the exclusive-OR of the sign bits
 // of the source and destination operands.
 PRIVATE inline uae_u32 FFPU get_quotient_sign(fpu_register const & ra, fpu_register const & rb)
 {
-	uae_u32 * const a = (uae_u32 *)&ra;
-	uae_u32 * const b = (uae_u32 *)&rb;
-	return (((a[FHI] ^ b[FHI]) & 0x80000000) ? FPSR_QUOTIENT_SIGN : 0);
+	fpu_register_parts const *a = (fpu_register_parts const *)&ra;
+	fpu_register_parts const *b = (fpu_register_parts const *)&rb;
+	return (((a->parts[FHI] ^ b->parts[FHI]) & 0x80000000) ? FPSR_QUOTIENT_SIGN : 0);
 }
 
 // Quotient Byte is loaded with the sign and least significant
@@ -402,13 +400,13 @@ PRIVATE inline fpu_register FFPU make_single(uae_u32 value)
 		return (0.0);
 	
 	fpu_register result;
-	uae_u32 * p = (uae_u32 *)&result;
+	fpu_register_parts * p = (fpu_register_parts *)&result;
 
 	uae_u32 sign = (value & 0x80000000);
 	uae_u32 exp  = ((value & 0x7F800000) >> 23) + 1023 - 127;
 
-	p[FLO] = value << 29;
-	p[FHI] = sign | (exp << 20) | ((value & 0x007FFFFF) >> 3);
+	p->parts[FLO] = value << 29;
+	p->parts[FHI] = sign | (exp << 20) | ((value & 0x007FFFFF) >> 3);
 
 	fpu_debug(("make_single (%X) = %.04f\n",value,(double)result));
 	
@@ -422,10 +420,10 @@ PRIVATE inline uae_u32 FFPU extract_single(fpu_register const & src)
 		return 0;
 	
 	uae_u32 result;
-	uae_u32 *p = (uae_u32 *)&src;
+	fpu_register_parts const *p = (fpu_register_parts const *)&src;
 
-	uae_u32 sign = (p[FHI] & 0x80000000);
-	uae_u32 exp  = (p[FHI] & 0x7FF00000) >> 20;
+	uae_u32 sign = (p->parts[FHI] & 0x80000000);
+	uae_u32 exp  = (p->parts[FHI] & 0x7FF00000) >> 20;
 
 	if(exp + 127 < 1023) {
 		exp = 0;
@@ -435,7 +433,7 @@ PRIVATE inline uae_u32 FFPU extract_single(fpu_register const & src)
 		exp = exp + 127 - 1023;
 	}
 
-	result = sign | (exp << 23) | ((p[FHI] & 0x000FFFFF) << 3) | (p[FLO] >> 29);
+	result = sign | (exp << 23) | ((p->parts[FHI] & 0x000FFFFF) << 3) | (p->parts[FLO] >> 29);
 
 	fpu_debug(("extract_single (%.04f) = %X\n",(double)src,result));
 
@@ -449,8 +447,8 @@ PRIVATE inline fpu_register FFPU make_extended(uae_u32 wrd1, uae_u32 wrd2, uae_u
 		return 0.0;
 	
 	fpu_register result;
-	uae_u32 *p = (uae_u32 *)&result;
-	
+	fpu_register_parts *p = (fpu_register_parts *)&result;
+
 	uae_u32 sign =  wrd1 & 0x80000000;
 	uae_u32 exp  = (wrd1 >> 16) & 0x7fff;
 
@@ -487,8 +485,8 @@ PRIVATE inline fpu_register FFPU make_extended(uae_u32 wrd1, uae_u32 wrd2, uae_u
 	}
 
 	// drop the explicit integer bit.
-	p[FLO] = (wrd2 << 21) | (wrd3 >> 11);
-	p[FHI] = sign | (exp << 20) | ((wrd2 & 0x7FFFFFFF) >> 11);
+	p->parts[FLO] = (wrd2 << 21) | (wrd3 >> 11);
+	p->parts[FHI] = sign | (exp << 20) | ((wrd2 & 0x7FFFFFFF) >> 11);
 
 	fpu_debug(("make_extended (%X,%X,%X) = %.04f\n",wrd1,wrd2,wrd3,(double)result));
 
@@ -532,9 +530,9 @@ PRIVATE inline void FFPU make_extended_no_normalize(
 	}
 
 	// drop the explicit integer bit.
-	uae_u32 *p = (uae_u32 *)&result;
-	p[FLO] = (wrd2 << 21) | (wrd3 >> 11);
-	p[FHI] = sign | (exp << 20) | ((wrd2 & 0x7FFFFFFF) >> 11);
+	fpu_register_parts *p = (fpu_register_parts *)&result;
+	p->parts[FLO] = (wrd2 << 21) | (wrd3 >> 11);
+	p->parts[FHI] = sign | (exp << 20) | ((wrd2 & 0x7FFFFFFF) >> 11);
 
 	fpu_debug(("make_extended (%X,%X,%X) = %.04f\n",wrd1,wrd2,wrd3,(float)(*(double *)p)));
 }
@@ -548,14 +546,14 @@ PRIVATE inline void FFPU extract_extended(fpu_register const & src,
 		*wrd1 = *wrd2 = *wrd3 = 0;
 		return;
 	}
-	
-	uae_u32 *p = (uae_u32 *)&src;
+
+	fpu_register_parts const *p = (fpu_register_parts const *)&src;
 	
 	fpu_debug(("extract_extended (%X,%X)\n",p[FLO],p[FHI]));
 
-	uae_u32 sign =  p[FHI] & 0x80000000;
+	uae_u32 sign =  p->parts[FHI] & 0x80000000;
 
-	uae_u32 exp  = ((p[FHI] >> 20) & 0x7ff);
+	uae_u32 exp  = ((p->parts[FHI] >> 20) & 0x7ff);
 	// Check for maximum
 	if(exp == 0x7FF) {
 		exp = 0x7FFF;
@@ -565,8 +563,8 @@ PRIVATE inline void FFPU extract_extended(fpu_register const & src,
 
 	*wrd1 = sign | (exp << 16);
 	// always set the explicit integer bit.
-	*wrd2 = 0x80000000 | ((p[FHI] & 0x000FFFFF) << 11) | ((p[FLO] & 0xFFE00000) >> 21);
-	*wrd3 = p[FLO] << 11;
+	*wrd2 = 0x80000000 | ((p->parts[FHI] & 0x000FFFFF) << 11) | ((p->parts[FLO] & 0xFFE00000) >> 21);
+	*wrd3 = p->parts[FLO] << 11;
 
 	fpu_debug(("extract_extended (%.04f) = %X,%X,%X\n",(double)src,*wrd1,*wrd2,*wrd3));
 }
@@ -578,9 +576,9 @@ PRIVATE inline fpu_register FFPU make_double(uae_u32 wrd1, uae_u32 wrd2)
 		return 0.0;
 	
 	fpu_register result;
-	uae_u32 *p = (uae_u32 *)&result;
-	p[FLO] = wrd2;
-	p[FHI] = wrd1;
+	fpu_register_parts *p = (fpu_register_parts *)&result;
+	p->parts[FLO] = wrd2;
+	p->parts[FHI] = wrd1;
 
 	fpu_debug(("make_double (%X,%X) = %.04f\n",wrd1,wrd2,(double)result));
 
@@ -598,9 +596,9 @@ PRIVATE inline void FFPU extract_double(fpu_register const & src,
 		return;
   }
 */
-	uae_u32 *p = (uae_u32 *)&src;
-	*wrd2 = p[FLO];
-	*wrd1 = p[FHI];
+	fpu_register_parts const *p = (fpu_register_parts const *)&src;
+	*wrd2 = p->parts[FLO];
+	*wrd1 = p->parts[FHI];
 
 	fpu_debug(("extract_double (%.04f) = %X,%X\n",(double)src,*wrd1,*wrd2));
 }
@@ -643,25 +641,25 @@ PRIVATE inline uae_u32 FFPU extract_single(fpu_register const & src)
 	fpu_register src0 = src;
 #endif
 
-  if (src == 0.0)
+	if (src == 0.0)
 		return 0;
-  if (src < 0) {
+	if (src < 0) {
 		tmp = 0x80000000;
 		src = -src;
-  } else {
+	} else {
 		tmp = 0;
-  }
-  frac = frexp (src, &expon);
-  frac += 0.5 / 16777216.0;
-  if (frac >= 1.0) {
+	}
+	frac = frexp (src, &expon);
+	frac += 0.5 / 16777216.0;
+	if (frac >= 1.0) {
 		frac /= 2.0;
 		expon++;
-  }
+	}
 	result = tmp | (((expon + 127 - 1) & 0xff) << 23) | (((int) (frac * 16777216.0)) & 0x7fffff);
 
 	// fpu_debug(("extract_single (%.04f) = %X\n",(float)src0,result));
 
-  return (result);
+	return (result);
 }
 
 // to exten
@@ -1225,7 +1223,7 @@ PRIVATE inline int FFPU fpp_cond(int condition)
 #if 0
 	return fpcctrue(condition);
 #else
-	switch (condition) {
+	switch (condition & 0x1f) {
 	case 0x00:	CONDRET("False",0);
 	case 0x01:	CONDRET("Equal",Z);
 	case 0x02:	CONDRET("Ordered Greater Than",!(NaN || Z || N));
@@ -1320,11 +1318,11 @@ void FFPU fpuop_scc(uae_u32 opcode, uae_u32 extra)
 		put_byte(ad, cc ? 0xff : 0x00);
 }
 
-void FFPU fpuop_trapcc(uae_u32 opcode, uaecptr oldpc)
+void FFPU fpuop_trapcc(uae_u32 opcode, uaecptr oldpc, uae_u32 extra)
 {
-	fpu_debug(("ftrapcc_opp %X at %08lx\n", (uae_u32)opcode, m68k_getpc ()));
+	fpu_debug(("ftrapcc_opp %X, %X at %08lx\n", (uae_u32)opcode, (uae_u32)extra, m68k_getpc ()));
 	
-	int cc = fpp_cond(opcode & 0x3f);
+	int cc = fpp_cond(extra & 0x3f);
 	if (cc == -1) {
 		m68k_setpc (oldpc);
 		op_illg (opcode);
