@@ -265,8 +265,17 @@ static void gen_set_fault_pc (void)
 /* getv == 1: fetch data; getv != 0: check for odd address. If movem != 0,
  * the calling routine handles Apdi and Aipi modes.
  * gb-- movem == 2 means the same thing but for a MOVE16 instruction */
-static void genamode (amodes mode, char *reg, wordsizes size, char *name, int getv, int movem, int xlateflag)
+
+/* fixup indicates if we want to fix up adress registers in pre decrement
+ * or post increment mode now (0) or later (1). A value of 2 will then be
+ * used to do the actual fix up. This allows to do all memory readings
+ * before any register is modified, and so to rerun operation without
+ * side effect in case a bus fault is generated by any memory access.
+ * XJ - 2006/11/13 */
+static void genamode2 (amodes mode, char *reg, wordsizes size, char *name, int getv, int movem, int xlateflag, int fixup)
 {
+    if (fixup != 2)
+    {
     if (!using_mmu)
 	xlateflag = XLATE_PHYS;
 
@@ -327,19 +336,19 @@ static void genamode (amodes mode, char *reg, wordsizes size, char *name, int ge
 	    if (movem)
 		printf ("\tuaecptr %sa = m68k_areg(regs, %s);\n", name, reg);
 	    else
-		printf ("\tuaecptr %sa = m68k_areg(regs, %s) -= areg_byteinc[%s];\n", name, reg, reg);
+		printf ("\tuaecptr %sa = m68k_areg(regs, %s) - areg_byteinc[%s];\n", name, reg, reg);
 	    break;
 	 case sz_word:
 	    if (movem)
 		printf ("\tuaecptr %sa = m68k_areg(regs, %s);\n", name, reg);
 	    else
-		printf ("\tuaecptr %sa = m68k_areg(regs, %s) -= 2;\n", name, reg);
+		printf ("\tuaecptr %sa = m68k_areg(regs, %s) - 2;\n", name, reg);
 	    break;
 	 case sz_long:
 	    if (movem)
 		printf ("\tuaecptr %sa = m68k_areg(regs, %s);\n", name, reg);
 	    else
-		printf ("\tuaecptr %sa = m68k_areg(regs, %s) -= 4;\n", name, reg);
+		printf ("\tuaecptr %sa = m68k_areg(regs, %s) - 4;\n", name, reg);
 	    break;
 	 default:
 	    abort ();
@@ -460,6 +469,12 @@ static void genamode (amodes mode, char *reg, wordsizes size, char *name, int ge
     if (!movem)
 	switch (mode) {
 	 case Aipi:
+	    if (fixup == 1)
+	    {
+		printf ("\tfixup.flag = 1;\n");
+		printf ("\tfixup.reg = %s;\n", reg);
+		printf ("\tfixup.value = m68k_areg(regs, %s);\n", reg);
+	    }
 	    switch (size) {
 	     case sz_byte:
 		printf ("\tm68k_areg(regs, %s) += areg_byteinc[%s];\n", reg, reg);
@@ -474,12 +489,37 @@ static void genamode (amodes mode, char *reg, wordsizes size, char *name, int ge
 		abort ();
 	    }
 	    break;
-/*	 case Apdi:
+	 case Apdi:
+	    if (fixup == 1)
+	    {
+		printf ("\tfixup.flag = 1;\n");
+		printf ("\tfixup.reg = %s;\n", reg);
+		printf ("\tfixup.value = m68k_areg(regs, %s);\n", reg);
+	    }
 	    printf ("\tm68k_areg (regs, %s) = %sa;\n", reg, name);
-	    break;*/
+	    break;
 	 default:
 	    break;
 	}
+
+    }
+    else /* (fixup != 2) */
+    {
+    if (!movem)
+	switch (mode) {
+	 case Aipi:
+	 case Apdi:
+	    printf ("\tfixup.flag = 0;\n");
+	    break;
+	 default:
+	    break;
+	}
+    }
+}
+
+static void genamode (amodes mode, char *reg, wordsizes size, char *name, int getv, int movem, int xlateflag)
+{
+	genamode2 (mode, reg, size, name, getv, movem, xlateflag, 0);
 }
 
 static void genastore (char *from, amodes mode, char *reg, wordsizes size, char *to, int xlateflag)
@@ -954,8 +994,9 @@ static void gen_opcode (unsigned long int opcode)
 	genastore ("newv", curi->dmode, "dstreg", sz_long, "dst", xlateflag);
 	break;
      case i_SUBX:
-	genamode (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 1);
 	genamode (curi->dmode, "dstreg", curi->size, "dst", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 2);
 	start_brace ();
 	printf ("\tuae_u32 newv = dst - src - (GET_XFLG ? 1 : 0);\n");
 	genflags (flag_subx, curi->size, "newv", "src", "dst");
@@ -963,8 +1004,9 @@ static void gen_opcode (unsigned long int opcode)
 	genastore ("newv", curi->dmode, "dstreg", curi->size, "dst", xlateflag);
 	break;
      case i_SBCD:
-	genamode (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 1);
 	genamode (curi->dmode, "dstreg", curi->size, "dst", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 2);
 	start_brace ();
 	printf ("\tuae_u16 newv_lo = (dst & 0xF) - (src & 0xF) - (GET_XFLG ? 1 : 0);\n");
 	printf ("\tuae_u16 newv_hi = (dst & 0xF0) - (src & 0xF0);\n");
@@ -994,8 +1036,9 @@ static void gen_opcode (unsigned long int opcode)
 	genastore ("newv", curi->dmode, "dstreg", sz_long, "dst", xlateflag);
 	break;
      case i_ADDX:
-	genamode (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 1);
 	genamode (curi->dmode, "dstreg", curi->size, "dst", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 2);
 	start_brace ();
 	printf ("\tuae_u32 newv = dst + src + (GET_XFLG ? 1 : 0);\n");
 	genflags (flag_addx, curi->size, "newv", "src", "dst");
@@ -1003,8 +1046,9 @@ static void gen_opcode (unsigned long int opcode)
 	genastore ("newv", curi->dmode, "dstreg", curi->size, "dst", xlateflag);
 	break;
      case i_ABCD:
-	genamode (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 1);
 	genamode (curi->dmode, "dstreg", curi->size, "dst", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 2);
 	start_brace ();
 	printf ("\tuae_u16 newv_lo = (src & 0xF) + (dst & 0xF) + (GET_XFLG ? 1 : 0);\n");
 	printf ("\tuae_u16 newv_hi = (src & 0xF0) + (dst & 0xF0);\n");
@@ -1146,8 +1190,9 @@ static void gen_opcode (unsigned long int opcode)
 	genastore ("val", curi->dmode, "dstreg", curi->size, "dst", xlateflag);
 	break;
      case i_MOVE:
-	genamode (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 1);
 	genamode (curi->dmode, "dstreg", curi->size, "dst", GENA_GETV_FETCH_ALIGN, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 2);
 	genflags (flag_logical, curi->size, "src", "", "");
 	genastore ("src", curi->dmode, "dstreg", curi->size, "dst", xlateflag);
 	break;
@@ -1272,8 +1317,8 @@ static void gen_opcode (unsigned long int opcode)
 	m68k_pc_offset = 0;
 	break;
      case i_RTD:
-	genamode (Aipi, "7", sz_long, "pc", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
 	genamode (curi->smode, "srcreg", curi->size, "offs", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode (Aipi, "7", sz_long, "pc", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
 	printf ("\tm68k_areg(regs, 7) += offs;\n");
 	printf ("\tm68k_setpc_rte(pc);\n");
 	fill_prefetch_0 ();
@@ -1281,10 +1326,10 @@ static void gen_opcode (unsigned long int opcode)
 	m68k_pc_offset = 0;
 	break;
      case i_LINK:
+	genamode (curi->dmode, "dstreg", curi->size, "offs", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
 	genamode (Apdi, "7", sz_long, "old", GENA_GETV_FETCH_ALIGN, GENA_MOVEM_DO_INC, XLATE_LOG);
 	genamode (curi->smode, "srcreg", sz_long, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
 	genastore ("m68k_areg(regs, 7)", curi->smode, "srcreg", sz_long, "src", xlateflag);
-	genamode (curi->dmode, "dstreg", curi->size, "offs", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
 	printf ("\tm68k_areg(regs, 7) += offs;\n");
 	genastore ("src", Apdi, "7", sz_long, "old", xlateflag);
 	break;
@@ -1300,14 +1345,16 @@ static void gen_opcode (unsigned long int opcode)
 	m68k_pc_offset = 0;
 	break;
      case i_TRAPV:
+	printf ("\tuaecptr oldpc = m68k_getpc();\n");
 	sync_m68k_pc ();
-	printf ("\tif (GET_VFLG) { Exception(7,m68k_getpc()); goto %s; }\n", endlabelstr);
+	printf ("\tif (GET_VFLG) { Exception(7,oldpc); goto %s; }\n", endlabelstr);
 	need_endlabel = 1;
 	break;
      case i_RTR:
 	printf ("\tMakeSR();\n");
-	genamode (Aipi, "7", sz_word, "sr", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (Aipi, "7", sz_word, "sr", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 1);
 	genamode (Aipi, "7", sz_long, "pc", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
+	genamode2 (Aipi, "7", sz_word, "sr", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG, 2);
 	printf ("\tregs.sr &= 0xFF00; sr &= 0xFF;\n");
 	printf ("\tregs.sr |= sr; m68k_setpc(pc);\n");
 	fill_prefetch_0 ();
@@ -2020,14 +2067,14 @@ static void gen_opcode (unsigned long int opcode)
 	printf ("\top_illg(opcode);\n");
 	break;
      case i_TRAPcc:
+	printf ("\tuaecptr oldpc = m68k_getpc();\n");
 	if (curi->smode != am_unknown && curi->smode != am_illg)
 	    genamode (curi->smode, "srcreg", curi->size, "dummy", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
-	printf ("\tif (cctrue(%d)) { Exception(7,m68k_getpc()); goto %s; }\n", curi->cc, endlabelstr);
+	sync_m68k_pc ();
+	printf ("\tif (cctrue(%d)) { Exception(7,oldpc); goto %s; }\n", curi->cc, endlabelstr);
 	need_endlabel = 1;
 	break;
      case i_DIVL:
-	sync_m68k_pc ();
-	start_brace ();
 	printf ("\tuaecptr oldpc = m68k_getpc();\n");
 	genamode (curi->smode, "srcreg", curi->size, "extra", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
 	genamode (curi->dmode, "dstreg", curi->size, "dst", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
@@ -2051,31 +2098,34 @@ static void gen_opcode (unsigned long int opcode)
 	genamode (curi->smode, "srcreg", curi->size, "extra", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
 	genamode (curi->dmode, "dstreg", sz_long, "dst", GENA_GETV_FETCH_ALIGN, GENA_MOVEM_DO_INC, XLATE_LOG);
 	start_brace ();
+	printf ("\tuae_u32 bdata[2];");
 	printf ("\tuae_s32 offset = extra & 0x800 ? m68k_dreg(regs, (extra >> 6) & 7) : (extra >> 6) & 0x1f;\n");
 	printf ("\tint width = (((extra & 0x20 ? m68k_dreg(regs, extra & 7) : extra) -1) & 0x1f) +1;\n");
 	if (curi->dmode == Dreg) {
-	    printf ("\tuae_u32 tmp = m68k_dreg(regs, dstreg) << (offset & 0x1f);\n");
+	    printf ("\tuae_u32 tmp = m68k_dreg(regs, dstreg);\n");
+	    printf ("\toffset &= 0x1f;\n");
+	    printf ("\ttmp = (tmp << offset) | (tmp >> (32 - offset));\n");
+	    printf ("\tbdata[0] = tmp & ((1 << (32 - width)) - 1);\n");
 	} else {
-	    printf ("\tuae_u32 tmp,bf0,bf1;\n");
-	    printf ("\tdsta += (offset >> 3) | (offset & 0x80000000 ? ~0x1fffffff : 0);\n");
-	    printf ("\tbf0 = get_long(dsta);bf1 = get_byte(dsta+4) & 0xff;\n");
-	    printf ("\ttmp = (bf0 << (offset & 7)) | (bf1 >> (8 - (offset & 7)));\n");
+	    printf ("\tuae_u32 tmp;\n");
+	    printf ("\tdsta += offset >> 3;\n");
+	    printf ("\ttmp = get_bitfield(dsta, bdata, offset, width);\n");
 	}
-	printf ("\ttmp >>= (32 - width);\n");
-	printf ("\tSET_NFLG_ALWAYS (tmp & (1 << (width-1)) ? 1 : 0);\n");
+	printf ("\tSET_NFLG_ALWAYS (((uae_s32)tmp) < 0 ? 1 : 0);\n");
+	if (curi->mnemo == i_BFEXTS)
+		printf ("\ttmp = (uae_s32)tmp >> (32 - width);\n");
+	else
+		printf ("\ttmp >>= (32 - width);\n");
 	printf ("\tSET_ZFLG (tmp == 0); SET_VFLG (0); SET_CFLG (0);\n");
 	switch (curi->mnemo) {
 	 case i_BFTST:
 	    break;
 	 case i_BFEXTU:
+	 case i_BFEXTS:
 	    printf ("\tm68k_dreg(regs, (extra >> 12) & 7) = tmp;\n");
 	    break;
 	 case i_BFCHG:
-	    printf ("\ttmp = ~tmp;\n");
-	    break;
-	 case i_BFEXTS:
-	    printf ("\tif (GET_NFLG) tmp |= width == 32 ? 0 : (-1 << width);\n");
-	    printf ("\tm68k_dreg(regs, (extra >> 12) & 7) = tmp;\n");
+	    printf ("\ttmp = tmp ^ (0xffffffffu >> (32 - width));\n");
 	    break;
 	 case i_BFCLR:
 	    printf ("\ttmp = 0;\n");
@@ -2086,10 +2136,11 @@ static void gen_opcode (unsigned long int opcode)
 	    printf ("\tm68k_dreg(regs, (extra >> 12) & 7) = offset;\n");
 	    break;
 	 case i_BFSET:
-	    printf ("\ttmp = 0xffffffff;\n");
+	    printf ("\ttmp = 0xffffffffu >> (32 - width);\n");
 	    break;
 	 case i_BFINS:
 	    printf ("\ttmp = m68k_dreg(regs, (extra >> 12) & 7);\n");
+	    printf ("\ttmp = tmp & (0xffffffffu >> (32 - width));\n");
 	    printf ("\tSET_NFLG_ALWAYS (tmp & (1 << (width - 1)) ? 1 : 0);\n");
 	    printf ("\tSET_ZFLG (tmp == 0);\n");
 	    break;
@@ -2099,26 +2150,12 @@ static void gen_opcode (unsigned long int opcode)
 	if (curi->mnemo == i_BFCHG
 	    || curi->mnemo == i_BFCLR
 	    || curi->mnemo == i_BFSET
-	    || curi->mnemo == i_BFINS)
-	{
-	    printf ("\ttmp <<= (32 - width);\n");
+	    || curi->mnemo == i_BFINS) {
 	    if (curi->dmode == Dreg) {
-		printf ("\tm68k_dreg(regs, dstreg) = (m68k_dreg(regs, dstreg) & ((offset & 0x1f) == 0 ? 0 :\n");
-		printf ("\t\t(0xffffffff << (32 - (offset & 0x1f))))) |\n");
-		printf ("\t\t(tmp >> (offset & 0x1f)) |\n");
-		printf ("\t\t(((offset & 0x1f) + width) >= 32 ? 0 :\n");
-		printf (" (m68k_dreg(regs, dstreg) & ((uae_u32)0xffffffff >> ((offset & 0x1f) + width))));\n");
+		printf ("\ttmp = bdata[0] | (tmp << (32 - width));\n");
+	        printf ("\tm68k_dreg(regs, dstreg) = (tmp >> offset) | (tmp << (32 - offset));\n");
 	    } else {
-		printf ("\tbf0 = (bf0 & (0xff000000 << (8 - (offset & 7)))) |\n");
-		printf ("\t\t(tmp >> (offset & 7)) |\n");
-		printf ("\t\t(((offset & 7) + width) >= 32 ? 0 :\n");
-		printf ("\t\t (bf0 & ((uae_u32)0xffffffff >> ((offset & 7) + width))));\n");
-		printf ("\tput_long(dsta,bf0 );\n");
-		printf ("\tif (((offset & 7) + width) > 32) {\n");
-		printf ("\t\tbf1 = (bf1 & (0xff >> (width - 32 + (offset & 7)))) |\n");
-		printf ("\t\t\t(tmp << (8 - (offset & 7)));\n");
-		printf ("\t\tput_byte(dsta+4,bf1);\n");
-		printf ("\t}\n");
+		printf ("\tput_bitfield(dsta, bdata, tmp, offset, width);\n");
 	    }
 	}
 	break;
@@ -2179,11 +2216,12 @@ static void gen_opcode (unsigned long int opcode)
 	sync_m68k_pc ();
 	start_brace ();
 	printf ("\tuaecptr oldpc = m68k_getpc();\n");
+	printf ("\tuae_u16 extra = %s;\n", gen_nextiword());
 	if (curi->smode != am_unknown && curi->smode != am_illg)
 	    genamode (curi->smode, "srcreg", curi->size, "dummy", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
 	sync_m68k_pc ();
 	swap_opcode ();
-	printf ("\tfpuop_trapcc(opcode, oldpc);\n");
+	printf ("\tfpuop_trapcc(opcode, oldpc, extra);\n");
 	break;
      case i_FBcc:
 	sync_m68k_pc ();
